@@ -130,11 +130,14 @@ file_info_new (const gchar *uri)
 void
 file_info_free (FileInfo *info)
 {
+  if (info)
+    {
 	if (info->uri) {
 		g_free (info->uri);
 	}
 
 	g_slice_free (FileInfo, info);
+    }
 }
 
 TaskInfo *
@@ -154,9 +157,12 @@ task_info_new (const gchar *mime)
 void
 task_info_add_uri (TaskInfo *info, gchar *uri)
 {
-	FileInfo *file;
-	file = file_info_new (uri);
-	info->files = g_slist_prepend (info->files, file);
+  if (info && uri && *uri)
+    {
+      FileInfo *file;
+      file = file_info_new (uri);
+      info->files = g_slist_prepend (info->files, file);
+    }
 }
 
 void
@@ -691,6 +697,8 @@ thumber_set_state (Thumber *thumber,
 		if (priv->idle_id == 0) {
 			priv->idle_id = g_idle_add (thumber_process_func, 
 						    thumber);
+			g_source_set_can_recurse (g_main_context_find_source_by_id (NULL, priv->idle_id),
+						  FALSE);
 		}
 		if (priv->quit_timeout_id != 0) {
 			g_source_remove (priv->quit_timeout_id);
@@ -791,6 +799,11 @@ thumber_populate_file_queue (Thumber *thumber, TaskInfo *task) {
 	}
 }
 
+/* Recurrency is not allowed so there is no risk of new item being started
+   before the previous is done even though the pipeline bus is polled
+   in the mainloop
+*/
+
 static gboolean
 thumber_process_func (gpointer data)
 {
@@ -802,8 +815,9 @@ thumber_process_func (gpointer data)
 	thumber = THUMBER (data);
 	priv = THUMBER_GET_PRIVATE (thumber);
 
-	if ((file = g_queue_peek_head (priv->file_queue)) != NULL) {
-		
+	if (priv && priv->pipe &&
+	    ((file = g_queue_pop_head (priv->file_queue)) != NULL)) {
+
 		if (!thumber_pipe_run (priv->pipe,
 				       file->uri,
 				       &error)) {
@@ -831,7 +845,6 @@ thumber_process_func (gpointer data)
 				       file->uri);
 		}
 
-		file = g_queue_pop_head (priv->file_queue);
 		file_info_free (file);
 			
 	} else {
